'Make the browser remember the mode selection (def/hier/comment) between class selections. There is also partially-working code (commented out) to remember the message category selection.'!

Browser addInstVarName: 'lastEdit'!
Browser addInstVarName: 'messageCategoryListSelection'!

!Browser methodsFor: 'accessing' stamp: 'stp 8/15/2000 14:21'!
contents
	"Depending on the current selection, different information is retrieved.
	Answer a string description of that information. This information is the
	method of the currently selected class and message."
	| comment theClass latestCompiledMethod |
	latestCompiledMethod _ currentCompiledMethod.
	currentCompiledMethod _ nil.

	editSelection == #none ifTrue: [^ ''].
	editSelection == #editSystemCategories 
		ifTrue: [^ systemOrganizer printString].
	theClass _ self selectedClass.
	editSelection == #newClass 
		ifTrue: [^ theClass
			ifNil: [Class template: self selectedSystemCategoryName]
			ifNotNil: [Class templateForSubclassOf: theClass 
							category: self selectedSystemCategoryName]].
	editSelection == #editComment 
		ifTrue: [theClass ifNil: [^ ''].
				comment _ theClass comment.
				comment size = 0
				ifTrue: [^ 'This class has not yet been commented.']
				ifFalse: [^ comment]].
	theClass _ self selectedClassOrMetaClass.
	theClass ifNotNil: [editSelection == #editClass 
		ifTrue: [^ theClass definitionST80: Preferences printAlternateSyntax not]].
	theClass ifNotNil: 
		[editSelection == #hierarchy 
			ifTrue: [^ theClass printHierarchy].
		editSelection == #editMessageCategories 
			ifTrue: [^ self classOrMetaClassOrganizer printString].
		editSelection == #newMessage
			ifTrue: [^ theClass sourceCodeTemplate]].
	editSelection == #editMessage
		ifTrue: [currentCompiledMethod _ latestCompiledMethod.
			"currentCompiledMethod ifNil: 
					[self halt. editSelection := #newMessage.
					^theClass sourceCodeTemplate]."
			^ self selectedMessage].
	editSelection == #byteCodes
		ifTrue: [^ (theClass compiledMethodAt: self selectedMessageName) symbolic asText].

	^''
	"self error: 'Browser internal error: unknown edit selection.'"!
]style[(8 1760)f1bcblue;,f1! !

!Browser methodsFor: 'accessing' stamp: 'stp 8/15/2000 03:57'!
contentsSelection
	"Return the interval of text in the code pane to select when I set the pane's contents"

	messageCategoryListIndex > 0 & (messageListIndex = 0) & (editSelection == #newMessage)
		ifTrue: [^ 1 to: 500]	"entire empty method template"
		ifFalse: [^ 1 to: 0]  "null selection"! !

!Browser methodsFor: 'system category list' stamp: 'stp 8/23/2000 14:08'!
systemCategoryListIndex: anInteger 
	"Set the selected system category index to be anInteger. Update all other 
	selections to be deselected."

	systemCategoryListIndex _ anInteger.
	classListIndex _ 0.
	messageCategoryListIndex _ 0.
	messageListIndex _ 0.
	lastEdit ifNil: [lastEdit := #editClass].
	(editSelection == #editMessage)
			ifTrue: [editSelection := lastEdit].
	metaClassIndicated _ false.
	self setClassOrganizer.
	contents _ nil.
	self changed: #systemCategorySelectionChanged.
	self changed: #systemCategoryListIndex.	"update my selection"
	self changed: #classList.
	self changed: #messageCategoryList.
	self changed: #messageList.
	self contentsChanged.
! !

!Browser methodsFor: 'system category functions' stamp: 'stp 8/15/2000 06:36'!
findClass
	"Search for a class by name."
	| pattern foundClass classNames index toMatch exactMatch potentialClassNames |

	self okToChange ifFalse: [^ self classNotFound].
	pattern _ FillInTheBlank request: 'Class name or fragment?'
				initialAnswer: ParagraphEditor clipboardText string
				centerAt: Sensor cursorPoint.
	pattern isEmpty ifTrue: [^ self classNotFound].
	toMatch _ (pattern copyWithout: $.) asLowercase.
	potentialClassNames _ self potentialClassNames asOrderedCollection.
	classNames _ pattern last = $. 
		ifTrue: [potentialClassNames select:
					[:nm |  nm asLowercase = toMatch]]
		ifFalse: [potentialClassNames select: 
					[:n | n includesSubstring: toMatch caseSensitive: false]].
	classNames isEmpty ifTrue: [^ self classNotFound].
	exactMatch _ classNames detect: [:each | each asLowercase = toMatch] ifNone: [nil].

	index _ classNames size = 1
		ifTrue:	[1]
		ifFalse:	[exactMatch
			ifNil: [(PopUpMenu labelArray: classNames lines: #()) startUp]
			ifNotNil: [classNames addFirst: exactMatch.
				(PopUpMenu labelArray: classNames lines: #(1)) startUp]].
	index = 0 ifTrue: [^ self classNotFound].
	foundClass _ Smalltalk at: (classNames at: index) asSymbol.
 	self selectCategoryForClass: foundClass.
	self selectClass: foundClass
! !

!Browser methodsFor: 'class functions' stamp: 'stp 8/23/2000 14:09'!
editClass
	"Retrieve the description of the class definition."

	classListIndex = 0 ifTrue: [^ self].
	self okToChange ifFalse: [^ self].
	self messageCategoryListIndex: 0.
	lastEdit := editSelection := #editClass.
	self changed: #editClass.
	self changed: #contents.
! !

!Browser methodsFor: 'class functions' stamp: 'stp 8/23/2000 14:09'!
editComment
	"Retrieve the description of the class comment."

	classListIndex = 0 ifTrue: [^ self].
	self okToChange ifFalse: [^ self].
	self messageCategoryListIndex: 0.
	lastEdit := editSelection := #editComment.
	self changed: #classSelectionChanged.
	self contentsChanged
! !

!Browser methodsFor: 'class functions' stamp: 'stp 8/15/2000 14:33'!
hierarchy
	"Display the inheritance hierarchy of the receiver's selected class."

	classListIndex = 0 ifTrue: [^ self].
	self okToChange ifFalse: [^ self].
	self messageCategoryListIndex: 0.
	lastEdit := editSelection := #hierarchy.
	self changed: #editComment.
	self contentsChanged.
	^ self! !

!Browser methodsFor: 'message category list' stamp: 'stp 8/15/2000 08:13'!
messageCategoryList
	"Answer the selected category of messages."

	classListIndex = 0
		ifTrue: [^ Array new]
		ifFalse: [^ (Array with: ClassOrganizer allCategory), 
						self classOrMetaClassOrganizer categories]! !

!Browser methodsFor: 'message category list' stamp: 'stp 8/15/2000 16:12'!
messageCategoryListIndex: anInteger 
	"Set the selected message category to be the one indexed by anInteger."

	messageCategoryListIndex _ anInteger.
"	Transcript show: 'ca: ', messageCategoryListIndex printString; cr."
	messageListIndex := 0.
	anInteger = 0
		ifTrue: [editSelection := lastEdit.
				messageCategoryListSelection := nil]
		ifFalse: [((lastEdit ~~ #newMessage) & (lastEdit ~~ #editMessage))
				ifFalse: [lastEdit := editSelection].
			editSelection := #newMessage.
		"	anInteger = 1
				ifTrue: [messageCategoryListSelection := nil]
				ifFalse: [messageCategoryListSelection := nil.
					messageCategoryListSelection := self messageCategoryListSelection] " ].
	contents := nil.
	self changed: #messageCategoryListIndex.	"update my selection"
	self changed: #messageList.
	self contentsChanged
! !

!Browser methodsFor: 'message category list' stamp: 'stp 8/15/2000 14:12'!
messageCategoryListSelection
	"Return the selected category name or nil."

	| sel lis |
	classListIndex = 0 ifTrue: [^nil].
						"select based on remembered protocol"
"	messageCategoryListSelection ifNotNil:
		[sel := self messageCategoryListSelectionIndex.
		sel ifNotNil: 
			[messageCategoryListIndex := sel.
			^self messageCategoryList at: sel]].
"
	lis := self messageCategoryList.
	^ ((lis size = 0 
		or: [messageCategoryListIndex = 0]) 
		or: [lis size < messageCategoryListIndex])
			ifTrue: [nil]
			ifFalse: [sel := lis at: (messageCategoryListIndex max: 1).
				"Transcript show: 'ms: ', sel; cr."
				messageCategoryListSelection := sel.
				^sel]! !

!Browser methodsFor: 'message category list' stamp: 'stp 8/15/2000 07:11'!
messageCategoryListSelectionIndex
	"Return the selected category name or nil."

	^messageCategoryListSelection isNil
		ifTrue: [self messageCategoryList 
					indexOf: messageCategoryListSelection 
					ifAbsent: [1]]
		ifFalse: [1]
! !

!Browser methodsFor: 'message list' stamp: 'stp 8/15/2000 08:14'!
messageList
	"Answer an Array of the message selectors of the currently selected message category, provided that the messageCategoryListIndex is in proper range.  Otherwise, answer an empty Array  If messageCategoryListIndex is found to be larger than the number of categories (it happens!!), it is reset to zero."

	| sel |
	sel _ self messageCategoryListSelection.
"	Transcript show: 'se: ', sel printString; cr."
	sel ifNil: [^ Array new].
	^ sel = ClassOrganizer allCategory
		ifTrue: [self classOrMetaClassOrganizer
				ifNil: [Array new]
				ifNotNil: [self classOrMetaClassOrganizer allMethodSelectors]]
		ifFalse: [(self classOrMetaClassOrganizer 
							listAtCategoryNumber: messageCategoryListIndex - 1)
				ifNil: [messageCategoryListIndex _ 0.  Array new]]! !

!Browser methodsFor: 'message list' stamp: 'stp 8/15/2000 14:15'!
messageListIndex: anInteger 
	"Set the selected message selector to be the one indexed by anInteger."

	messageListIndex _ anInteger.
	editSelection _ 
		anInteger = 0
			ifTrue: [#newMessage]
			ifFalse: [#editMessage].
	contents _ nil.
	self changed: #messageListIndex.	"update my selection"
	self contentsChanged! !

!Browser methodsFor: 'class list' stamp: 'stp 8/23/2000 14:37'!
classListIndex: anInteger 
	"Set anInteger to be the index of the current class selection."

	| className |
	classListIndex := anInteger.
	self setClassOrganizer.
	messageCategoryListIndex := 0.
	messageListIndex := 0.
	lastEdit ifNil: [lastEdit := #editClass].
	((editSelection == #editMessage) | (editSelection == #none) 
			| (editSelection == #newMessage))
		ifTrue: [editSelection := lastEdit].
	contents := nil.
	self selectedClass ifNotNil:		"update recent class list"
		[className := self selectedClass name.
					(RecentClasses includes: className)
				ifTrue: [RecentClasses remove: className].
			RecentClasses addFirst: className.
			RecentClasses size > RecentSize
				ifTrue: [RecentClasses removeLast].
									"if no inst methods, switch to metaclass"
		metaClassIndicated := metaClassIndicated 
							| classOrganizer allMethodSelectors isEmpty].
	self changed: #classSelectionChanged.
	self changed: #classListIndex.	"update my selection"
	self changed: #messageCategoryList.
"STP -- Select the 'all' protocol by default"
	classListIndex = 0
		ifFalse: [messageCategoryListIndex := 
			(self messageCategoryListSelection == nil)
				ifTrue: [1]
				ifFalse: [self messageCategoryListSelectionIndex].
			messageListIndex _ 0.
			self changed: #messageCategoryListIndex.
			self changed: #messageList]
		ifTrue: [self changed: #messageList].
	self changed: #contents

"	self classCommentIndicated
		ifTrue: []
		ifFalse: [editSelection := anInteger = 0
					ifTrue: [metaClassIndicated | (systemCategoryListIndex == 0)
						ifTrue: [#none]
						ifFalse: [#newClass]]
					ifFalse: [#editClass]].
"! !


